@@ -3,7 +3,9 @@ from sgqlc.operation import Operation
 
 from politylink.graphql import POLITYLINK_AUTH, POLITYLINK_URL
 from politylink.graphql.schema import *
-from politylink.graphql.schema import _UrlInput, _BillInput, _SpeechInput, _MinutesInput, _CommitteeInput, _NewsInput
+from politylink.graphql.schema import _UrlInput, _BillInput, _SpeechInput, _MinutesInput, _CommitteeInput, _NewsInput, \
+    _TimelineInput, _NewsFilter, _Neo4jDateTimeInput, _BillFilter, _CommitteeFilter, _MinutesFilter, _SpeechFilter, \
+    _TimelineFilter, _UrlFilter
 
 MAX_BATCH_SIZE = 100
 
@@ -91,29 +93,74 @@ class GraphQLClient:
             ret = self.exec(op)
         return ret
 
-    def get_all_bills(self):
+    def get(self, id_):
+        """
+        General method to get single GraphQL object by id
+        """
+
+        op = self.build_get_operation(id_)
+        res = self.endpoint(op)
+        self.validate_response_or_raise(res)
+        cls = id_.split(':')[0].lower()
+        data = getattr(op + res, cls)
+        if len(data) == 0:
+            raise GraphQLException(f'{id_} does not exist')
+        elif len(data) == 1:
+            return data[0]
+        else:
+            raise GraphQLException(f'multiple {id_} exist')
+
+    def get_all_bills(self, fields=None):
         """
         Special method to get all Bills
         :return: list of Bills
         """
 
-        op = self.build_all_bills_operation()
-        data = self.endpoint(op)
-        self.validate_response_or_raise(data)
-        bills = (op + data).bill
-        return bills
+        if fields is None:
+            fields = ['id', 'name', 'billNumber']
+        op = self.build_all_bills_operation(fields)
+        res = self.endpoint(op)
+        self.validate_response_or_raise(res)
+        return (op + res).bill
 
-    def get_all_committees(self):
+    def get_all_committees(self, fields=None):
         """
         Special method to get all Committees
         :return: list of Committees
         """
 
-        op = self.build_all_committees_operation()
-        data = self.endpoint(op)
-        self.validate_response_or_raise(data)
-        committees = (op + data).committee
-        return committees
+        if fields is None:
+            fields = ['id', 'name']
+        op = self.build_all_committees_operation(fields)
+        res = self.endpoint(op)
+        self.validate_response_or_raise(res)
+        return (op + res).committee
+
+    def get_all_minutes(self, fields=None):
+        """
+        Special method to get all Minutes
+        :return: list of Minutes
+        """
+
+        if fields is None:
+            fields = ['id', 'name']
+        op = self.build_all_minutes_operation(fields)
+        res = self.endpoint(op)
+        self.validate_response_or_raise(res)
+        return (op + res).minutes
+
+    def get_all_news(self, fields=None, start_date=None, end_date=None):
+        """
+        Special method to get all News
+        :return: list of News
+        """
+
+        if fields is None:
+            fields = ['id', 'title', 'published_at']
+        op = self.build_all_news_operation(fields, start_date, end_date)
+        res = self.endpoint(op)
+        self.validate_response_or_raise(res)
+        return (op + res).news
 
     @staticmethod
     def validate_response_or_raise(res):
@@ -121,21 +168,43 @@ class GraphQLClient:
             raise GraphQLException(res['errors'])
 
     @staticmethod
-    def build_all_bills_operation():
+    def build_all_bills_operation(fields):
         op = Operation(Query)
         bills = op.bill()
-        bills.id()
-        bills.name()
-        bills.bill_number()
+        for field in fields:
+            getattr(bills, field)()
         return op
 
     @staticmethod
-    def build_all_committees_operation():
+    def build_all_committees_operation(fields):
         op = Operation(Query)
         committees = op.committee()
-        committees.id()
-        committees.name()
-        committees.aliases()
+        for field in fields:
+            getattr(committees, field)()
+        return op
+
+    @staticmethod
+    def build_all_minutes_operation(fields):
+        op = Operation(Query)
+        minutes = op.minutes()
+        for field in fields:
+            getattr(minutes, field)()
+        return op
+
+    @staticmethod
+    def build_all_news_operation(fields, start_date=None, end_date=None):
+        def to_neo4j_datetime(dt):
+            return _Neo4jDateTimeInput(year=dt.year, month=dt.month, day=dt.day)
+
+        op = Operation(Query)
+        news_filter = _NewsFilter(None)
+        if start_date:
+            news_filter.published_at_gte = to_neo4j_datetime(start_date)
+        if end_date:
+            news_filter.published_at_lt = to_neo4j_datetime(end_date)
+        news = op.news(filter=news_filter)
+        for field in fields:
+            getattr(news, field)()
         return op
 
     @staticmethod
@@ -157,6 +226,8 @@ class GraphQLClient:
             res = op.merge_committee(**param)
         elif isinstance(obj, Speech):
             res = op.merge_speech(**param)
+        elif isinstance(obj, Timeline):
+            res = op.merge_timeline(**param)
         else:
             raise GraphQLException(f'unknown object type to merge: {type(obj)}')
         res.id()
@@ -219,8 +290,48 @@ class GraphQLClient:
                 to=_CommitteeInput({'id': to_id}),
                 __alias__=maybe_alias
             )
+        elif from_id.startswith('Bill') and to_id.startswith('Timeline'):
+            res = op.merge_timeline_bills(
+                from_=_BillInput({'id': from_id}),
+                to=_TimelineInput({'id': to_id}),
+                __alias__=maybe_alias
+            )
+        elif from_id.startswith('Minutes') and to_id.startswith('Timeline'):
+            res = op.merge_timeline_minutes(
+                from_=_MinutesInput({'id': from_id}),
+                to=_TimelineInput({'id': to_id}),
+                __alias__=maybe_alias
+            )
+        elif from_id.startswith('News') and to_id.startswith('Timeline'):
+            res = op.merge_timeline_news(
+                from_=_NewsInput({'id': from_id}),
+                to=_TimelineInput({'id': to_id}),
+                __alias__=maybe_alias
+            )
         else:
             raise GraphQLException(f'unknown id types to link: from={from_id} to={to_id}')
         res.from_.id()
         res.to.id()
         return op
+
+    @staticmethod
+    def build_get_operation(id_):
+        op = Operation(Query)
+        cls = id_.split(':')[0].lower()
+        if cls == 'bill':
+            op.bill(filter=_BillFilter({'id': id_}))
+        elif cls == 'committee':
+            op.committee(filter=_CommitteeFilter({'id': id_}))
+        elif cls == 'minutes':
+            op.minutes(filter=_MinutesFilter({'id': id_}))
+        elif cls == 'speech':
+            op.speech(filter=_SpeechFilter({'id': id_}))
+        elif cls == 'url':
+            op.url(filter=_UrlFilter({'id': id_}))
+        elif cls == 'news':
+            op.news(filter=_NewsFilter({'id': id_}))
+        elif cls == 'timeline':
+            op.timeline(filter=_TimelineFilter({'id': id_}))
+        else:
+            raise GraphQLException(f'unknown id type : {id_}')
+        return op
